
package com.jmrosengard.xmlbroker;

import com.jmrosengard.xmlbroker.os.*;
import java.io.*;
import java.util.*;

/**
*   Loads the configuration file, redirects <i>stdout</i> and <i>stderr</i> to log files.
*   The following properties <i>must</i> be defined:
*   <ul><li>xmlbroker.conf (as a -D option to the java interpreter)</li>
*   <li>xmlbroker.outLog (in the configuration files)</li>
*   <li>xmlbroker.errLog (in the configuration files)</li></ul>
*   All necessary objects are then created (via factories) and pooled.
*/
public class Main{
 public static final String FILE_SEP=System.getProperty("file.separator");
 private NativeHelper helper=new NativeHelper(); // Java wrapper for C shared lib.
 private ResourceBundle xb_props;
 private long gid;

 public Main(){
  InputStream in=null;
  OutputStream out=null;
  OutputStream err=null;
  try{
   // load config file
   String path=System.getProperty("xmlbroker.home")+System.getProperty("file.separator");
   in=ClassLoader.getSystemResourceAsStream(System.getProperty("xmlbroker.conf"));
   xb_props=new PropertyResourceBundle(in);
   // redirect stdout & stderr to logs
   System.setOut(new PrintStream(out=new BufferedOutputStream(
    new FileOutputStream(xb_props.getString("xmlbroker.outLog"),true))));
   System.setErr(new PrintStream(err=new BufferedOutputStream(
    new FileOutputStream(xb_props.getString("xmlbroker.errLog"),true))));
   // initialise global ID
   gid=helper.getGIDSeed();
   // log instance startup OK
   System.out.println("\n--- XMLbroker instance \""+
    xb_props.getString("xmlbroker.instance")+"\" started on "+new Date().toString()+" ---"
    +"\n GID seed: "+Long.toString(gid,16));
  }catch(Exception e){
   e.printStackTrace();
  }
  finally{    // cleanup whatever happens
   try{
    if(in!=null)in.close();
    if(out!=null){
     out.flush();
     out.close();
    }
    if(err!=null){
     err.flush();
     err.close();
    }
   }catch(IOException e){}
  }
 }
 /**
 * Returns the next unique <i>Global ID</i> in the GID sequence
 */
 private synchronized String nextGID(){
  return Long.toString(++gid,16);
 }
 /**
 * Creates an instance of XMLbroker
 */
 public static void main(String[] args){
 new Main();
 }
}
