
package com.jmrosengard.xmlbroker.broker;

import com.jmrosengard.xmlbroker.deploy.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
/**
*  SOAP server connector to the HTTP protocol.
*  This implementation is based upon the SOAP-HTTP binding specifications (SOAP 1.1)
*  and the RFC 2376 and covers:
*  <ul>
*  <li>the request/response media-type</li>
*  <li>the request/response HTTP method</li>
*  <li>the SOAP-specific HTTP header field</li>
*  </ul>
*/

abstract class HttpConnector extends HttpServlet{

 public static final String SOAP_MEDIA_TYPE="text/xml";
 public static final String SOAP_HTTP_HEADER_SOAPACTION="SOAPAction";
 public static final String SOAP_HTTP_METHOD="POST";
 private ResourceBundle xb_props;

 /**
 *  Set the HTTP POST response as a SOAP response
 *  and delegates the request-specific logic to
 *  the doSoapPost() <i>hook method</i>--according
 *  to the <i>Template Method Pattern</i> (see
 *  [Gamma, E. et al. 1995]).
 */
 public void doPost(HttpServletRequest req,HttpServletResponse resp)
 throws ServletException,IOException{
  resp.setContentType(SOAP_MEDIA_TYPE);
  String targetUri=req.getHeader(SOAP_HTTP_HEADER_SOAPACTION);
  try{
   PrintWriter outBuf=resp.getWriter();
   doSoapPost(targetUri,req.getReader(),outBuf); // call hook method
   outBuf.flush();
  }catch(SoapReqException e){
   e.printStackTrace();
  }
 }

 /**
 *  This hook method must be overriden by
 *  all subclasses to provide a request-specific
 *  implementation.
 */
 public abstract void doSoapPost(String targetUri,BufferedReader inBuf,PrintWriter outBuf)
  throws ServletException,IOException,SoapReqException;

 /**
 * Initialise XMLbroker server
 */
 public void init(){
  DeployManager man=new DeployManager();
  man.load();  // load services
 }

 /**
 *  The HTTP GET method is <i>not</i> used in the
 *  SOAP-HTTP binding. doGet() therefore provides
 *  a welcome message--only intended to test whether
 *  XMLbroker is running, with a simple browser.
 */
 public void doGet(HttpServletRequest req,HttpServletResponse resp)
 throws IOException,ServletException{
  resp.setContentType("text/html");
  PrintWriter outBuf=resp.getWriter();
  outBuf.print("<html><body>"+(new Date()).toString()+"<hr><h2>XMLbroker is ready for service.<hr></h2></body></html>");
  outBuf.flush();
 }
}
