
package com.jmrosengard.xmlbroker.broker;

import java.util.*;

/**
* Registers and deregisters Port implementations. Each port
* represents an HTTP-level view of a service.
* All threads access a single instance. This is efforced using
* an adaptation of the Singleton Pattern [Gamma,E. et al. 1995].
* Note that the methods are not <tt>synchronized</tt> since
* Hashtable is itself <tt>synchronized</tt>.
*/

public final class PortRegistry{
 protected Hashtable ports;
 private static PortRegistry reg;

 private PortRegistry(){
  ports=new Hashtable(10);
 }
 /**
 * Registers a port
 */
 public void register(PortKey portKey,Port port){
  ports.put(portKey,port);
 }
  /**
 * Deregisters a port. Does nothing if the key does not
 * exist in the Hashtable.
 */
 public void deregister(PortKey portKey){
  ports.remove(portKey);
 }
 /**
 * The <i>only</i> way to create an instance of PortRegistry
 * is to use this method.
 */
 public static PortRegistry getDefaultRegistry(){
  return reg=reg!=null?reg:new PortRegistry();
 }
 /**
 * Returns a reference to the instance Port of port matching
 * the string in portKey.
 */
 public Port getPort(PortKey portKey){
  return (Port)ports.get(portKey);
 }
}
