package com.jmrosengard.xmlbroker.javaipr;

import java.lang.reflect.*;
import java.util.*;
import com.jmrosengard.xmlbroker.soapipr.Mode;
import com.jmrosengard.xmlbroker.soapipr.Parameter;
import com.jmrosengard.xmlbroker.soapipr.SoapInterpreterException;
import com.jmrosengard.xmlbroker.deploy.XbService;
import com.jmrosengard.xmlbroker.deploy.XbString;

/**
* Interprets the a method according to the following convension:
* <ul><li>Parameter type allowed: only XbString</li>
* <li>return type must be `void'--with no inconvenience since `out' parameters
* are supported</li>
* <li>the method name contains the passing mode marks (in or out)</li></ul>
* e.g. void foo__out__in(String param1, String param2)
*/
public class JavaInterpreter{
 private Class cls;
 private Method method;
 private ArrayList modes;
 private Class[] params;
 private ModeQueueBuilder builder;

 protected void validate() throws JavaIprException{
  try{
   Method[] methods=cls.getMethods();
   /**
   * each interface must declare a SINGLE method
   */
   if(methods.length!=1) throw new JavaIprException(
    cls.getName()+" : each interface MUST declare a SINGLE method");
   method=methods[0];
   params=method.getParameterTypes();
   /**
   * only the <tt>String</tt> formal parameter type is supported
   */
   for(int i=0;i<params.length;i++)
    if(params[i]!=XbString.class)throw new JavaIprException(
     cls.getName()+" : only the `XbString' formal parameter type is supported");
   /**
   * only <tt>void</tt> return type is allowed
   * note that the void keyword is reified in
   * the Void class.
   */
   if(methods[0].getReturnType()!=void.class) throw new JavaIprException(
    cls.getName()+" : only the `void' return type is allowed");
   /**
   * the method name MUST contain the passing mode marks, all `ins' before
   * `outs'-- the markers are `__in' and `__out' and MUST qualify the parameters
   * in the order they are entered.
   */
   builder=new ModeQueueBuilder(method.getName());
   modes=builder.getModes();
   // check number of mode marks == number of parameters
   if(modes.size()!=params.length) throw new JavaIprException(
    cls.getName()
    +" : the number of mode marks is different from the number of parameters");
  }
  catch(Exception e){
   e.printStackTrace();
  }
 }
 public JavaInterpreter(Class cls){
  this.cls=cls;
 }
 /**
 * Creates a com.jmrosengard.xmlbroker.soapipr.Method object from
 * a java <tt>interface</tt>.
 */
 public synchronized com.jmrosengard.xmlbroker.soapipr.Method buildMethod(){
  com.jmrosengard.xmlbroker.soapipr.Method meth=null;
  try{
   validate();
   meth=new com.jmrosengard.xmlbroker.soapipr.Method(builder.getMethodName());
   for(int i=0;i<modes.size();i++)
    meth.setParam(new Parameter((Mode)modes.get(i)));
  }
  catch(JavaIprException e){
   e.printStackTrace();
  }
  catch(SoapInterpreterException e){
   e.printStackTrace();
  }
  return meth;
 }

 /**
 * Only for testing
 */
 public static void main(String[] a){
  JavaInterpreter ji=new  JavaInterpreter(deployed.NtpService.class);
  ji.buildMethod();
 }
}