package com.jmrosengard.xmlbroker.soapipr;

import com.jmrosengard.xmlbroker.broker.PortKey;
import com.jmrosengard.xmlbroker.deploy.XbString;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import java.util.*;

/**
* Generates the method-specific part of a SOAP response
*/

public class RespMsg extends Message{
 private PrintWriter outbuf;
 private String methodName;
 private ArrayList params;
 private String serviceNs;
 private String serviceNsUri;

 public RespMsg(PrintWriter outbuf, String methodName, PortKey pk){
  super(outbuf);
  this.outbuf=outbuf;
  this.methodName=methodName+"Response";
  serviceNs=pk.toString();
  serviceNsUri=XB_NS_URI_BASE+serviceNs;
 }

 /**
 * Hook method (See Template Pattern in [Gamma,E. & al. 1995])
 * that generates the actual SOAP response message content.
 */
 protected void doContent(){
  meth=doc.createElementNS(serviceNsUri,serviceNs+':'+methodName);
  meth.setAttributeNS(XML_NS_URI,"xmlns:"+serviceNs,serviceNsUri);
  body.appendChild(meth);
  meth.appendChild(doc.createTextNode("\n"));
  // add `in' parameters
  if(params!=null)
   for(Iterator it=params.iterator();it.hasNext();){
    Parameter p=(Parameter)it.next();
    param=doc.createElementNS(serviceNsUri,serviceNs+':'+p.getName());
    param.appendChild(doc.createTextNode(p.getValue().toString()));
    meth.appendChild(param);
    meth.appendChild(doc.createTextNode("\n"));
   }
  body.appendChild(doc.createTextNode("\n"));
 }

 /**
 * Set the list of parameters (e.g. `out' parameters)
 */
 public void setParams(ArrayList params){
  this.params=params;
 }

 /**
 * Only for testing
 */
 public static void main(String[] a) throws com.jmrosengard.xmlbroker.broker.SoapReqException{
  RespMsg m=new RespMsg(null,"myTestMethod",new PortKey("http://www.jmrosengard.com/xmlbroker#test"));
  ArrayList al=new ArrayList();
  al.add(new Parameter("param1",new Mode(Mode.MODE_OUT),new XbString("hello")));
  al.add(new Parameter("param2",new Mode(Mode.MODE_OUT),new XbString("XML world !")));
  m.setParams(al);
  m.build();
 }
}