package com.jmrosengard.xmlbroker.soapipr;

import java.io.*;
import java.util.*;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.apache.xerces.parsers.*;
import org.xml.sax.InputSource;
import com.jmrosengard.xmlbroker.deploy.XbString;


/**
* Encapsulates the XML parser (Apache Xerces)--
* creates a Method object, and a set of Parameter objects
* from a SOAP message.
*/

public class SoapInterpreter{

 DOMParser parser;
 Document message;
 BufferedReader inbuf;
 Method method;

 public SoapInterpreter(BufferedReader inbuf){
  this.inbuf=inbuf;
 }
 /**
 * Parse the SOAP message into a DOM tree, and
 * call buildMethod()
 */
 public Method parse(){
  try{
   parser=new DOMParser();
   parser.parse(new InputSource(inbuf));
   message=parser.getDocument();
   // <SOAP-ENV:Envelope> MUST be the first child element of a SOAP document
   if(! message.getFirstChild().getNodeName().equals("SOAP-ENV:Envelope"))
   throw new SoapInterpreterException("Invalid SOAP message: the element SOAP-ENV:Envelope is not present or mis-placed.");
   buildMethod(message);
  }
  catch(Exception e){
   e.printStackTrace();
  }
  return method;
 }

 /**
 * Recursive method-- builds a Method object
 */
 private void buildMethod(Node node){
  int type = node.getNodeType();
  // The serialised method's name is the first child element of <SOAP-ENV:Body>
  if(type==Node.ELEMENT_NODE && node.getNodeName().equals("SOAP-ENV:Body")){
   Node methodNode;
   for(methodNode=node.getFirstChild();
       methodNode.getNodeType()!=Node.ELEMENT_NODE;
       methodNode=methodNode.getNextSibling());
   // the message is a SOAP response if the method name contains `Response'
   boolean isRequest=methodNode.getLocalName().indexOf("Response")==-1;
   // create a Method object and a set of parameters
   String methodName=methodNode.getLocalName();
   method=new Method(
    isRequest?methodName:methodName.substring(0,methodName.length()-8));
    try{
     for(Node paramNode=methodNode.getFirstChild();
         paramNode!=null;
         paramNode=paramNode.getNextSibling())
      if(paramNode.getNodeType()==Node.ELEMENT_NODE)
      method.setParam(new Parameter(paramNode.getLocalName(),
                      new Mode(isRequest?Mode.MODE_IN:Mode.MODE_OUT),
                      new XbString(paramNode.getFirstChild().getNodeValue())));
     // Log parsed method
     StringBuffer paramList=new StringBuffer(40);
     Iterator it=method.paramIterator();
     Parameter p;
     while(it.hasNext()) paramList.append(
      "[").append(
      (p=(Parameter)it.next()).getName()).append(
      " ").append(
      p.getMode()).append(
      " = ").append(
      p.getValue().toString()).append("]");
     System.out.println("[XMLbroker] method parsed: "+
                        method.getName()+"("+paramList.toString()+")");
    }
    catch(SoapInterpreterException e){
     e.printStackTrace();
    }
  }
  //recurse
  for(Node child=node.getFirstChild();
  child!=null;child=child.getNextSibling()) buildMethod(child);
 }

 /**
 * Only for testing
 */
 /*
 public static void main(String args[]){
  (new SoapInterpreter()).parse();
 }*/
}